import type { LanguageInput, ThemedToken, ThemeInput, ThemeRegistrationAny, TokensResult } from '@shikijs/types'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useMemo, useState } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'

const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface SyntaxHighlighterProps {
  text: string
  language?: LanguageInput
  languageId?: string
  theme: ThemeRegistrationAny
  fontSize?: number
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const engine = createJavaScriptRegexEngine({
  forgiving: true,
  simulation: true,
  cache: new Map(),
})

function processThemeTokenColors(theme: ThemeRegistrationAny) {
  const tokenColorMap = new Map<string, { foreground?: string, fontStyle?: string }>()

  if (!theme.tokenColors)
    return tokenColorMap

  // Sort token colors from most specific to least specific, handling undefined scopes
  const sortedTokenColors = [...theme.tokenColors].sort((a, b) => {
    const aScope = Array.isArray(a.scope) ? a.scope[0] : a.scope ?? ''
    const bScope = Array.isArray(b.scope) ? b.scope[0] : b.scope ?? ''
    return (bScope?.split('.').length || 0) - (aScope?.split('.').length || 0)
  })

  sortedTokenColors.forEach((tokenColor) => {
    // Skip if no scope or settings
    if (!tokenColor.scope || !tokenColor.settings)
      return

    // Convert to array if it's a string
    const scopes = Array.isArray(tokenColor.scope) ? tokenColor.scope : [tokenColor.scope]

    scopes.forEach((scope) => {
      if (!scope)
        return // Skip empty scopes

      // Split compound scopes and store each part
      const scopeParts = scope.split(/[\s,]+/).filter(Boolean) // Remove empty strings
      scopeParts.forEach((part) => {
        if (!tokenColorMap.has(part)) {
          tokenColorMap.set(part, {
            foreground: tokenColor.settings.foreground,
            fontStyle: tokenColor.settings.fontStyle,
          })
        }
      })
    })
  })

  return tokenColorMap
}

function parseColor(color?: string): string | undefined {
  if (!color)
    return undefined

  // Remove any whitespace and standardize format
  color = color.trim().toLowerCase()

  // Handle rgba/rgb format
  if (color.startsWith('rgb'))
    return color

  // Handle hex colors
  if (color.startsWith('#')) {
    // Convert 3-digit hex to 6-digit
    if (color.length === 4) {
      const r = color[1]
      const g = color[2]
      const b = color[3]
      return `#${r}${r}${g}${g}${b}${b}`
    }

    // Handle 8-digit hex (with alpha)
    if (color.length === 9) {
      const r = Number.parseInt(color.slice(1, 3), 16)
      const g = Number.parseInt(color.slice(3, 5), 16)
      const b = Number.parseInt(color.slice(5, 7), 16)
      const a = (Number.parseInt(color.slice(7, 9), 16) / 255).toFixed(2)
      return `rgba(${r}, ${g}, ${b}, ${a})`
    }

    // Handle 6-digit hex
    if (color.length === 7) {
      const r = Number.parseInt(color.slice(1, 3), 16)
      const g = Number.parseInt(color.slice(3, 5), 16)
      const b = Number.parseInt(color.slice(5, 7), 16)
      return `rgb(${r}, ${g}, ${b})`
    }
  }

  return color
}

async function createHighlighter({ langs, themes }: { langs: LanguageInput[], themes: ThemeInput[] }) {
  if (!langs?.length) {
    throw new Error('Please provide languages when creating a highlighter.')
  }

  try {
    const highlighter = await createHighlighterCore({
      langs,
      themes,
      engine,
    })

    return highlighter
  }
  catch (error) {
    console.error('Failed to create highlighter:', error)
    throw error
  }
}

interface TokenScopeExplanation {
  scopeName: string
  themeMatches?: Array<{
    name?: string
    scope?: string
    settings: {
      foreground?: string
      fontStyle?: string
    }
  }>
}

interface TokenExplanation {
  content: string
  scopes: TokenScopeExplanation[]
}

interface TokenProps {
  token: ThemedToken & {
    explanation?: TokenExplanation[]
  }
  fontSize: number
  textStyle?: StyleProp<TextStyle>
  baseStyle: TextStyle
  tokenColorMap: Map<string, { foreground?: string, fontStyle?: string }>
}

// Token component moved outside SyntaxHighlighter
const Token = React.memo<TokenProps>(({ token, fontSize, textStyle, baseStyle, tokenColorMap }) => {
  const tokenStyle = useMemo(() => {
    let defaultForeground: string | undefined
    let defaultFontStyle: string | undefined

    // Find matching token color from the theme with proper type safety
    if (token.explanation?.[0]?.scopes) {
      for (const scope of token.explanation[0].scopes) {
        const themeToken = tokenColorMap.get(scope.scopeName)
        if (themeToken) {
          defaultForeground = themeToken.foreground
          defaultFontStyle = themeToken.fontStyle
          break
        }

        // Also check theme matches if available
        const themeMatches = scope.themeMatches
        if ((themeMatches ?? []).length > 0) {
          // Get the first match safely
          const firstMatch = (themeMatches ?? [])[0]
          if (firstMatch && firstMatch.settings) {
            const { settings } = firstMatch
            if (settings.foreground) {
              defaultForeground = settings.foreground
            }
            if (settings.fontStyle) {
              defaultFontStyle = settings.fontStyle
            }
          }
        }
      }
    }

    const style: TextStyle = {
      color: parseColor(token.color || defaultForeground),
      fontStyle: token.fontStyle === 2 || defaultFontStyle === 'italic' ? 'italic' : 'normal',
      fontWeight: token.fontStyle === 1 || defaultFontStyle === 'bold' ? '700' : '400',
      textDecorationLine: 'none',
      textAlign: 'left',
      fontSize,
    }

    return StyleSheet.flatten([baseStyle, style, textStyle])
  }, [token, fontSize, baseStyle, textStyle, tokenColorMap])

  return (
    <Text style={tokenStyle} selectable>
      {token.content}
    </Text>
  )
})

Token.displayName = 'Token'

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<Error | null>(null)

  // Process theme colors once
  const tokenColorMap = useMemo(() => processThemeTokenColors(theme), [theme])

  const styles = useMemo(() => {
    const backgroundColor = parseColor(theme.colors?.['editor.background']) || '#2e3440'
    const foregroundColor = parseColor(theme.colors?.['editor.foreground']) || '#d8dee9'

    console.log('[SyntaxHighlighter] Theme colors:', {
      background: backgroundColor,
      foreground: foregroundColor,
      allColors: theme.colors,
    })

    return StyleSheet.create({
      scrollView: {
        flex: 1,
        minHeight: 20,
      },
      codeContainer: {
        padding: 16,
        minWidth: '100%',
        backgroundColor,
      },
      lineContainer: {
        flexDirection: 'row',
        flexWrap: 'wrap',
      },
    })
  }, [theme])

  const baseStyle = useMemo(
    () => ({
      fontFamily: FONT_FAMILY,
      color: parseColor(theme.colors?.['editor.foreground']) || '#d8dee9',
    }),
    [theme],
  )

  useEffect(() => {
    let mounted = true

    async function highlight() {
      if (!text) {
        setTokens([])
        return
      }

      try {
        console.log('[SyntaxHighlighter] Creating highlighter for language:', languageId)
        const highlighter = await createHighlighter({
          langs: language ? [language] : [],
          themes: [Array.isArray(theme) ? theme[0] : theme],
        })

        console.log(
          '[SyntaxHighlighter] Theme token colors sample:',
          theme.tokenColors?.slice(0, 5).map(tc => ({
            scope: tc.scope,
            settings: {
              ...tc.settings,
              foreground: tc.settings.foreground,
              fontStyle: tc.settings.fontStyle,
            },
          })),
        )

        const result = highlighter.codeToTokens(text, {
          lang: languageId || 'plaintext',
          theme: Array.isArray(theme) ? theme[0] : theme,
          includeExplanation: true,
        }) as TokensResult

        if (mounted && result.tokens) {
          setTokens(result.tokens)
          setError(null)
        }
      }
      catch (err) {
        console.error('Syntax highlighting failed:', err)
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'))
          setTokens([[{ content: text, color: baseStyle.color, offset: 0 }]])
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme, baseStyle.color])

  if (error) {
    return (
      <ScrollView horizontal {...scrollViewProps} style={[styles.scrollView, scrollViewProps?.style]} contentContainerStyle={[styles.codeContainer, containerStyle]}>
        <Text style={[baseStyle, { fontSize }, textStyle]}>{text}</Text>
      </ScrollView>
    )
  }

  return (
    <ScrollView horizontal {...scrollViewProps} style={[styles.scrollView, scrollViewProps?.style]} contentContainerStyle={[styles.codeContainer, containerStyle]}>
      <View>
        {tokens.map((line, lineIndex) => (
          <View key={lineIndex} style={styles.lineContainer}>
            {line.map((token, tokenIndex) => {
              // Cast token to include the explanation type
              const tokenWithExplanation = token as ThemedToken & { explanation?: TokenExplanation[] }
              return <Token key={`${lineIndex}-${tokenIndex}`} token={tokenWithExplanation} fontSize={fontSize} textStyle={textStyle} baseStyle={baseStyle} tokenColorMap={tokenColorMap} />
            })}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default React.memo(SyntaxHighlighter)
